﻿using System.Threading.Tasks;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using NSubstitute;
using Robware.Api.Auth.Controllers;
using Robware.Auth.API;
using System;
using Xunit;

namespace Robware.Api.Auth.Tests.Controllers {
	public class ApiControllerTests {
		[Fact]
		public async Task Validate_WithValidKey_ReturnsOk() {
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			apiKeyValidator.Validate("key").Returns(true);
			var apiKeyRepository = Substitute.For<IApiKeys>();
			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Validate("key")).Should().BeOfType<OkResult>();
		}

		[Fact]
		public async Task Validate_WithInvalidKey_ReturnsUnauthorised() {
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			apiKeyValidator.Validate("key").Returns(false);
			var apiKeyRepository = Substitute.For<IApiKeys>();
			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Validate("key")).Should().BeOfType<UnauthorizedResult>();
		}

		[Fact]
		public async Task Create_WithName_ReturnsApiKeyJson() {
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			var apiKeyRepository = Substitute.For<IApiKeys>();

			var expectedKey = new ApiKey { 
				Name = "test",
				Key = "test",
				IssueTimestamp = DateTime.Now,
				Enabled = true
			};

			apiKeyRepository.Create("test").Returns(expectedKey);

			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Create("test")).Value.Should().BeEquivalentTo(expectedKey);
		}

		[Fact]
		public async Task List_ReturnsAllApiKeys() { 
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			var apiKeyRepository = Substitute.For<IApiKeys>();

			var apiKeys = new[] {
				new ApiKey {Name = "test1", Key = "test1", IssueTimestamp = DateTime.Now, Enabled = true},
				new ApiKey {Name = "test2", Key = "test2", IssueTimestamp = DateTime.Now, Enabled = false},
				new ApiKey {Name = "test3", Key = "test3", IssueTimestamp = DateTime.Now, Enabled = true}
			};

			apiKeyRepository.GetAll().Returns(apiKeys);

			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.List()).Value.Should().BeEquivalentTo(apiKeys);
		}

		[Fact]
		public async Task Delete_WithSuccessfulDeletion_ReturnsNoContent() { 
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			var apiKeyRepository = Substitute.For<IApiKeys>();

			apiKeyRepository.Delete("test").Returns(true);
			
			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Delete("test")).Should().BeOfType<NoContentResult>();
		}

		[Fact]
		public async Task Delete_WithUnsuccessfulDeletion_ReturnsBadRequest() {
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			var apiKeyRepository = Substitute.For<IApiKeys>();

			apiKeyRepository.Delete("test").Returns(false);

			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Delete("test")).Should().BeOfType<BadRequestResult>();
		}

		[Fact]
		public async Task Disable_WithSuccessfulDisablement_ReturnsNoContent() { 
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			var apiKeyRepository = Substitute.For<IApiKeys>();

			var apiKey = new ApiKey { 
				Name = "test",
				Key = "test",
				IssueTimestamp = DateTime.Now,
				Enabled = true
			};

			apiKeyRepository.Get("test").Returns(apiKey);
			apiKeyRepository.Update(apiKey).Returns(true);
			
			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Disable("test")).Should().BeOfType<NoContentResult>();
			apiKey.Enabled.Should().BeFalse();
		}

		[Fact]
		public async Task Disable_WithUnsuccessfulDisablement_ReturnsBadRequest() {
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			var apiKeyRepository = Substitute.For<IApiKeys>();

			var apiKey = new ApiKey { 
				Name = "test",
				Key = "test",
				IssueTimestamp = DateTime.Now,
				Enabled = true
			};

			apiKeyRepository.Get("test").Returns(apiKey);
			apiKeyRepository.Update(apiKey).Returns(false);

			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Disable("test")).Should().BeOfType<BadRequestResult>();
		}

		[Fact]
		public async Task Enable_WithSuccessfulEnablement_ReturnsNoContent() { 
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			var apiKeyRepository = Substitute.For<IApiKeys>();

			var apiKey = new ApiKey { 
				Name = "test",
				Key = "test",
				IssueTimestamp = DateTime.Now,
				Enabled = false
			};

			apiKeyRepository.Get("test").Returns(apiKey);
			apiKeyRepository.Update(apiKey).Returns(true);
			
			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Enable("test")).Should().BeOfType<NoContentResult>();
			apiKey.Enabled.Should().BeTrue();
		}

		[Fact]
		public async Task Enable_WithUnsuccessfulEnablement_ReturnsBadRequest() {
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			var apiKeyRepository = Substitute.For<IApiKeys>();

			var apiKey = new ApiKey { 
				Name = "test",
				Key = "test",
				IssueTimestamp = DateTime.Now,
				Enabled = false
			};

			apiKeyRepository.Get("test").Returns(apiKey);
			apiKeyRepository.Update(apiKey).Returns(false);

			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Enable("test")).Should().BeOfType<BadRequestResult>();
		}
	}
}